@@ -1,15 +1,16 @@
 from __future__ import annotations
 
-import re
+import shutil
 import typing as t
 from pathlib import Path
 
-import commonmark
-import commonmark.render.renderer
-from jinja2 import Environment, FileSystemLoader, select_autoescape
+if t.TYPE_CHECKING:
+    import commonmark
+    import commonmark.render.renderer
+    from jinja2 import Environment
 
-from . import helpers
 from .core import Context, Step
+from .dependencies import Dependency, import_install_check
 
 
 class JinjaRenderStep(Step):
@@ -18,6 +19,12 @@ class JinjaRenderStep(Step):
     """
     env: Environment
 
+    @classmethod
+    def get_dependencies(cls) -> set[Dependency]:
+        return super().get_dependencies() | {
+            Dependency('jinja2', 'pip', import_install_check),
+        }
+
     def __init__(self, env: Environment | None):
         self._temporary_env = env
 
@@ -31,17 +38,26 @@ class JinjaRenderStep(Step):
         if self._temporary_env:
             self.env = self._temporary_env
         else:
+            from jinja2 import Environment, FileSystemLoader, select_autoescape
             self.env = Environment(
                 loader=FileSystemLoader(context['input_dir']),
                 autoescape=select_autoescape()
             )
 
-    def render_template(self, template_name: str, meta: dict[str, t.Any], target_path: Path):
+    def render_template(self, template_name: str, meta: dict[str, t.Any], output_paths: list[Path]):
+        """
+        Look up a Jinja template by name and render it with @meta as
+        parameters, then save the result to each of the provided @output_paths.
+        """
+        if not output_paths:
+            return
+
         template = self.env.get_template(template_name)
-        target_path.parent.mkdir(parents=True, exist_ok=True)
-        template.stream(**meta).dump(str(target_path), encoding='utf-8')
-        # Returning the path makes for easier composition.
-        return target_path
+        for path in output_paths:
+            path.parent.mkdir(parents=True, exist_ok=True)
+        template.stream(**meta).dump(str(output_paths[0]), encoding='utf-8')
+        for path in output_paths[1:]:
+            shutil.copy(output_paths[0], path)
 
 
 class JinjaMarkdownStep(JinjaRenderStep):
@@ -51,6 +67,12 @@ class JinjaMarkdownStep(JinjaRenderStep):
     """
     encoding = 'utf-8'
 
+    @classmethod
+    def get_dependencies(cls) -> set[Dependency]:
+        return super().get_dependencies() | {
+            Dependency('commonmark', 'pip', import_install_check),
+        }
+
     def __init__(self,
                  default_template: str | None = None,
                  md_parser: commonmark.Parser | None = None,
@@ -58,20 +80,27 @@ class JinjaMarkdownStep(JinjaRenderStep):
                  jinja_env: Environment | None = None):
         super().__init__(jinja_env)
         self.default_template = default_template
-        self.md_parser = md_parser or commonmark.Parser()
-        self.md_renderer = md_renderer or commonmark.HtmlRenderer()
 
-    def __call__(self, path: Path, match: re.Match[str]) -> t.Iterable[Path]:
+        if md_parser:
+            self.md_parser = md_parser
+        else:
+            import commonmark
+            self.md_parser = commonmark.Parser()
+        if md_renderer:
+            self.md_renderer = md_renderer
+        else:
+            import commonmark
+            self.md_renderer = commonmark.HtmlRenderer()
+
+    def __call__(self, path: Path, output_paths: list[Path]):
         meta, content = self.extract_metadata(path.read_text(self.encoding))
         ast = self.md_parser.parse(content.strip())
         meta |= {'rendered_markdown': self.md_renderer.render(ast).strip()}
 
-        target_path = helpers.to_output(self.context, path, match, '.html')
-
-        yield self.render_template(
+        self.render_template(
             meta.get('template', self.default_template),
             meta,
-            target_path
+            output_paths
         )
 
     def extract_metadata(self, text: str):
